""" This class represents all warp-ledstripes combined. Starting with cybars 0th pixel up to the 9th sphere. More blikenfoo will be added in the future"""

import blinkenfoo
from Color import Color

class Blinkenroom:

    def __init__(self, p_device_list, p_fps = 42):

        self.fps = p_fps
        self.device_list = p_device_list
        self._calc_total_length()
        self._generate_index_map()
        self.previous_device = None
        self._remove_unreachable_devices()

    def _remove_unreachable_devices(self):
        for device in self.device_list:
            if not device.is_reachable():
                del self.device_list[self.device_list.index(device)]

    def set_rgb_frame(self, p_rgb_frame, p_offset = 0, p_fps_indicator = 0, p_fps_setter = None):
        frame_length = len(p_rgb_frame)

        if not frame_length == self.total_length:
            raise ValueError("Frame length does not match total length: Given %d, expected %d" % (frame_length, self.total_length))
        for i in range(frame_length):
            current_index = (i + p_offset) % frame_length
            r, g, b = p_rgb_frame[current_index]
            if not p_fps_setter == None and p_fps_indicator == current_index:
                self.set_led(i, r, g, b, p_fps_setter)
            else:
                self.set_led(i, r, g, b)
        self.flush()

    def set_full_color(self, p_color):
        for device in self.device_list:
            device.set_full_color(p_color)

    def set_led(self, p_index, p_r, p_g, p_b, p_fps_setter = None):
       device_index = self.global_map[p_index]
       device = self.device_list[device_index]
       if device == None:
           raise ValueError("Index is not mapped to any device (out of range)")

       if not p_fps_setter == None:
          if not self.previous_device == device:
              fps = device.calc_fps(self.fps)
              p_fps_setter(fps)

       for i in range(device_index):
          p_index -= self.device_list[i].length
       device.set_led(p_index, (p_r, p_g, p_b))

    def flush(self, p_reset = True):
        for device in self.device_list:
            device.flush(p_reset)

    def _calc_total_length(self):
        self.total_length = 0
        for device in self.device_list:
            self.total_length += device.length
        return self.total_length

    def _generate_index_map(self):
        self.global_map = [0 for _ in range(self.total_length)]

        current_index = 0
        for d in range(len(self.device_list)):
            device = self.device_list[d]
            for i in range(device.length):
                self.global_map[current_index + i] = d
            current_index += device.length


#warp_lounge = Blinkenroom(blinkenfoo.panel, blinkenfoo.spheres, blinkenfoo.warp_sign)
